(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2005-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: communication.mli,v 1.17 2017/05/31 15:54:22 fclement Exp $ *)

(** {3 The definition of communications}
 
 Communications are the elementary information units that are exchanged within
 the [Pio] library: each read or write operation has such an entity as
 argument. *)

type values = Typed_value.t array;;

type service_name =
  | Pio_Ok
  | Pio_Ko
  | Pio_Allo
  | Pio_Bye
  | Pio_Start
  | Pio_Stop
;;

type task_name = string;;

type t =
  | Pio_Phrase of values
  | Pio_Task of (task_name * values)
  | Pio_Result of values
  | Pio_Error of values
  | Pio_Service of (service_name * values)
and communication = t
(** The general type for communications. *)
;;

exception Wrong_communication of Upervasives.message;;

val service_name_of_string : string -> service_name
(** Convertion from [string] to [service_name]. *)
;;

val string_of_service_name : service_name -> string
(** Convertion from [service_name] to [string]. *)
;;
