(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: misc_sprint.ml,v 1.11 2017/11/13 12:40:10 fclement Exp $ *)

type 'a printer = 'a Og_types.printer;;

type 'a string_printer = string -> 'a -> string;;
type ('a, 'b) string_printer_lifter = 'a printer -> 'b string_printer;;

let printer_Couple pp_a pp_b ctx ppf = function
  | (a_s, b_s) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    pp_a Og_precedence.Ck_tuple_item ppf a_s;
    Format.fprintf ppf "%,,@ ";
    pp_b Og_precedence.Ck_tuple_item ppf b_s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"
;;

let printer_Triple pp_a pp_b pp_c ctx ppf = function
  | (a_s, b_s, c_s) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    pp_a Og_precedence.Ck_tuple_item ppf a_s;
    Format.fprintf ppf "%,,@ ";
    pp_b Og_precedence.Ck_tuple_item ppf b_s;
    Format.fprintf ppf "%,,@ ";
    pp_c Og_precedence.Ck_tuple_item ppf c_s;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"
;;

let format_sprint p ident x =
  let ppf = Format.str_formatter in
  Format.fprintf ppf "@[<hv 4>%s =@ %a@]"
    ident (p Og_precedence.Ck_toplevel) x;
  Format.flush_str_formatter ()
;;

let sprint_nat = format_sprint Unatural_print.printer_t;;
let sprint_int = format_sprint Og_printer.printer_Int;;
let sprint_float = format_sprint Og_printer.printer_Float;;

let sprint_homogeneous_couple p = format_sprint (printer_Couple p p);;
let sprint_list p = format_sprint (Og_printer.printer_List p);;
let sprint_list1 p = format_sprint (Ulist1_print.printer_list1 p);;
let sprint_array p = format_sprint (Og_printer.printer_Array p);;
let sprint_array_array p = sprint_array (Og_printer.printer_Array p);;
let sprint_matrix p = format_sprint (Umatrix_print.printer_t p);;

let sprint_nat_couple = sprint_homogeneous_couple Unatural_print.printer_t;;
let sprint_nat_array = sprint_array Unatural_print.printer_t;;
let sprint_nat_array_array = sprint_array_array Unatural_print.printer_t;;
let sprint_nat_matrix = sprint_matrix Unatural_print.printer_t;;

let sprint_int_array = sprint_array Og_printer.printer_Int;;
let sprint_int_array_array = sprint_array_array Og_printer.printer_Int;;
let sprint_int_matrix = sprint_matrix Og_printer.printer_Int;;

let sprint_float_array = sprint_array Og_printer.printer_Float;;
let sprint_float_array_array = sprint_array_array Og_printer.printer_Float;;
let sprint_float_matrix = sprint_matrix Og_printer.printer_Float;;

let sprint_select_array = sprint_array Upervasives_print.printer_select;;
let sprint_select_array_array =
  sprint_array_array Upervasives_print.printer_select
;;
let sprint_select_matrix = sprint_matrix Upervasives_print.printer_select;;

let sprint_color = format_sprint Colors_print.printer_t;;
