(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uaffine_plane.ml,v 1.8 2017/11/07 20:57:39 fclement Exp $ *)

(* Shortcuts for the Ulib library. *)

module Nat = Unatural;;

(* Affine plane operations. *)

module Make =
  functor (S : Uaffine_plane_spec.STRUCTURE) ->
struct
  type component = S.t;;

  type point = component * component;;
  type vector = component * component;;

  let origin = (S._0_, S._0_);;
  let null = (S._0_, S._0_);;

  let add (a, b) (u, v) = (S.add a u, S.add b v);;
  let sub (a1, b1) (a2, b2) = (S.sub a1 a2, S.sub b1 b2);;
end
;;

module Natural_structure =
struct
  type t = Nat.t;;

  let _0_ = Nat._0_
  and _1_ = Nat._1_
  ;;

  let add = Nat.add
  and sub = Nat.sub
  ;;
end
;;

module Natural = Make (Natural_structure);;

module Integer_structure =
struct
  type t = int;;

  let _0_ = 0
  and _1_ = 1
  ;;

  let add = Pervasives.( + )
  and sub = Pervasives.( - )
  ;;
end
;;

module Integer = Make (Integer_structure);;

module Float_structure =
struct
  type t = float;;

  let _0_ = 0.
  and _1_ = 1.
  ;;

  let add = Pervasives.( +. )
  and sub = Pervasives.( -. )
  ;;
end
;;

module Float = Make (Float_structure);;
