(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uassoc_list.ml,v 1.9 2017/11/07 20:57:39 fclement Exp $ *)

(* Shortcuts for the Ulib library. *)

module L = Ulist;;

(* Association list operations. *)

let increasing_order = Pervasives.compare;;
let decreasing_order x y = - (Pervasives.compare x y);;

type ('key, 'value) t = ('key * 'value) list;;
type ('key, 'value) assoc_list = ('key, 'value) t

(* Sorting. *)

let sort_key compare al =
  L.sort (fun (k1, _) (k2, _) -> compare k1 k2) al
;;

let sort_value compare al =
  L.sort (fun (_, v1) (_, v2) -> compare v1 v2) al
;;

let sort_key_increasing al = sort_key increasing_order al
and sort_key_decreasing al = sort_key decreasing_order al
;;

let sort_value_increasing al = sort_value increasing_order al
and sort_value_decreasing al = sort_value decreasing_order al
;;

(* Selection. *)

let item_with_best_key compare = function
  | [] -> raise Not_found
  | x0 :: al ->
      L.fold_left
        (fun ((ak, _) as accu) ((k, _) as x) ->
           if compare ak k < 0 then x else accu)
        x0 al
;;

let item_with_best_value compare = function
  | [] -> raise Not_found
  | x0 :: al ->
      L.fold_left
        (fun ((_, av) as accu) ((_, v) as x) ->
           if compare av v < 0 then x else accu)
        x0 al
;;

let item_with_highest_key al = item_with_best_key increasing_order al
and item_with_smallest_key al = item_with_best_key decreasing_order al
;;

let item_with_highest_value al = item_with_best_value increasing_order al
and item_with_smallest_value al = item_with_best_value decreasing_order al
;;

let first proj n al = L.map proj (L.first_p n al);;

let first_keys n al = first fst n al
and first_values n al = first snd n al
;;

let values_with_smallest_keys n al = first_values n (sort_value_increasing al)
and values_with_highest_keys n al = first_values n (sort_value_decreasing al)
;;

let keys_with_smallest_values n al = first_keys n (sort_value_increasing al)
and keys_with_highest_values n al = first_keys n (sort_value_decreasing al)
;;
