(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uassoc_list.mli,v 1.3 2017/05/31 17:27:34 fclement Exp $ *)

(** {3 Functions on association lists} *)

type ('key, 'value) t = ('key * 'value) list;;
type ('key, 'value) assoc_list = ('key, 'value) t

(** {6 Sorting} *)

val sort_key :
  ('key -> 'key -> int) -> ('key, 'value) t -> ('key, 'value) t
;;

val sort_value :
  ('value -> 'value -> int) -> ('key, 'value) t -> ('key, 'value) t
;;

val sort_key_increasing : ('key, 'value) t -> ('key, 'value) t;;
val sort_key_decreasing : ('key, 'value) t -> ('key, 'value) t;;
val sort_value_increasing : ('key, 'value) t -> ('key, 'value) t;;
val sort_value_decreasing : ('key, 'value) t -> ('key, 'value) t;;

(** {6 Selection} *)

val item_with_best_key :
  ('key -> 'key -> int) -> ('key, 'value) t -> 'key * 'value
;;

val item_with_best_value :
  ('value -> 'value -> int) -> ('key, 'value) t -> 'key * 'value
;;

val item_with_highest_key : ('key, 'value) t -> 'key * 'value;;
val item_with_smallest_key : ('key, 'value) t -> 'key * 'value;;
val item_with_highest_value : ('key, 'value) t -> 'key * 'value;;
val item_with_smallest_value : ('key, 'value) t -> 'key * 'value;;

val first_keys : Unatural.t -> ('key, 'value) t -> 'key list;;
val first_values : Unatural.t -> ('key, 'value) t -> 'value list;;

val values_with_smallest_keys : Unatural.t -> ('key, 'value) t -> 'value list;;
val values_with_highest_keys : Unatural.t -> ('key, 'value) t -> 'value list;;
val keys_with_smallest_values : Unatural.t -> ('key, 'value) t -> 'key list;;
val keys_with_highest_values : Unatural.t -> ('key, 'value) t -> 'key list;;
