(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: main.ml,v 1.23.2.1 2017/11/17 16:41:40 fclement Exp $ *)

(* Shortcuts for the Ulib library. *)

module Nat = Unatural;;
module Co = Ucommand_option;;

(* Shortcuts for the Refinement library. *)

module Opt = Options;;
module Config = Configuration;;

(* Main sequential program.
   Beware: we cannot use Config module to get the dimension (of type Nat.t),
   for it is not set yet. Hence the use of Opt module to get the dimension
   (of type integer). *)

let main () =
  Co.parse_phase Opt.phase_dimension;
  let dim = Nat.of_int (Opt.get_option_dimension ()) in
  let module Comps = (val Components.vector_of_nat dim) in
  let module R = Refinement.Make (Scalar.Float) (Comps) in
  R.algorithm R.compute_exact_indicator_on_selected_refinements;
  if Config.get_report_allocated_memory () then
    begin
      let ppf = Config.get_formatter () in
      let (tam, mu) = Monitor.total_allocated_memory () in
      Format.fprintf ppf "@.Total allocated memory: %g %s@." tam mu;
    end;
  exit 0;
;;

main ();;
