(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: unumbering.mli,v 1.4 2017/05/31 17:27:40 fclement Exp $*)

(** {3 Numbering bijections}

 A numbering bijection is a bijective function from a range of natural numbers
 \[0, n\[ to any type. For sake of efficiency, a numbering is explicitely
 represented as two reciprocal functions. *)

type length = Upervasives.length;;

type 'a value = 'a Uindex.t;;
type 'key h_dir = 'key -> 'key value;;
type 'key h_inv = 'key value -> 'key;;

type 'key t =
  {
   length : length;
   h_dir : 'key h_dir;
   h_inv : 'key h_inv;
  }
;;

val make : length -> 'key h_dir -> 'key h_inv -> 'key t;;

(** {6 Access functions} *)

val length : 'key t -> Upervasives.length;;

val value_of_key : 'key t -> 'key -> 'key value;;

val key_of_value : 'key t -> 'key value -> 'key;;

val equal : 'key t -> 'key t -> bool
(** Structural equality of generic numbering hash tables. *)
;;

(** {6 Iterators} *)

val iter : ('key -> 'key value -> unit) -> 'key t -> unit;;

val sub_iter :
  ('key -> 'key value -> unit) ->
  'key t -> 'key value -> Upervasives.length -> unit
;;

val fold_left :
  ('accu -> 'key -> 'key value -> 'accu) ->
  'accu -> 'key t -> 'accu
;;
