(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 4.2+dev0 generated code. *)

open Refinement_types_print;;

let rec printer_length ctx ppf = function
  | u_l -> Upervasives_print.printer_length ctx ppf u_l

and printer_width ctx ppf = function | l -> printer_length ctx ppf l

and printer_height ctx ppf = function | l -> printer_length ctx ppf l

and printer_size ctx ppf = function
  | (w, h) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    printer_width Og_precedence.Ck_tuple_item ppf w;
    Format.fprintf ppf "%,,@ ";
    printer_height Og_precedence.Ck_tuple_item ppf h;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"

and printer_tangential ctx ppf = function | l -> printer_length ctx ppf l

and printer_normal ctx ppf = function | l -> printer_length ctx ppf l

and printer_polar_size ctx ppf = function
  | (t, n) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    printer_tangential Og_precedence.Ck_tuple_item ppf t;
    Format.fprintf ppf "%,,@ ";
    printer_normal Og_precedence.Ck_tuple_item ppf n;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"
;;

let printer_color ctx ppf = function
  | c_t -> Colors_print.printer_t ctx ppf c_t
;;

let printer_font_name ctx ppf = function
  | s -> Og_printer.printer_String ctx ppf s
;;

let printer_aspect _ctx ppf = function
  | Display_types.Portrait ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Portrait"
  | Display_types.Landscape ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Landscape"
;;

let printer_direction _ctx ppf = function
  | Display_types.Horizontal_dir ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Horizontal_dir"
  | Display_types.Vertical_dir ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Vertical_dir"
;;

let printer_comps_direction _ctx ppf = function
  | Display_types.Horizontal_comps_dir ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Horizontal_comps_dir"
  | Display_types.Vertical_comps_dir ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Vertical_comps_dir"
;;

let rec printer_image_size ctx ppf = function | s -> printer_size ctx ppf s

and printer_font_size ctx ppf = function | s -> printer_size ctx ppf s

and printer_display_image_size ctx ppf = function
  | s -> printer_size ctx ppf s

and printer_window_size ctx ppf = function | s -> printer_size ctx ppf s

and printer_sub_window_size ctx ppf = function
  | w -> printer_window_size ctx ppf w

and printer_min_window_size ctx ppf = function
  | w -> printer_window_size ctx ppf w

and printer_max_window_size ctx ppf = function
  | w -> printer_window_size ctx ppf w
;;

let printer_scaling_factor ctx ppf = function
  | u_t -> Upositive_rational_print.printer_t ctx ppf u_t
;;

let printer_scaling ctx ppf = function
  | f_u_t_u_t ->
    Og_printer.printer_Fun Unatural_print.printer_t Unatural_print.printer_t
      ctx
      ppf
      f_u_t_u_t
;;

let printer_sub_window_sizes ctx ppf = function
  | {
      Display_types.title_sub_window_height = l;
      Display_types.legend_sub_window_polar_size = p;
      Display_types.colorbar_size = s;
      Display_types.colorbar_sub_window_tangential = l0;
      Display_types.colormap_sub_window_tangential = l1;
      Display_types.pixmap_sub_window_size = s0;
      Display_types.quantity_sub_window_size = s1;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,@[<hv 2>title_sub_window_height =";
    printer_length Og_precedence.Ck_record_field ppf l;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>legend_sub_window_polar_size =";
    printer_polar_size Og_precedence.Ck_record_field ppf p;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>colorbar_size =";
    printer_sub_window_size Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>colorbar_sub_window_tangential =";
    printer_length Og_precedence.Ck_record_field ppf l0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>colormap_sub_window_tangential =";
    printer_length Og_precedence.Ck_record_field ppf l1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>pixmap_sub_window_size =";
    printer_sub_window_size Og_precedence.Ck_record_field ppf s0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>quantity_sub_window_size =";
    printer_sub_window_size Og_precedence.Ck_record_field ppf s1;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_graphics_elements ctx ppf = function
  | {
      Display_types.comps_direction = c;
      Display_types.scaling_factor = s;
      Display_types.display_image_size = d;
      Display_types.window_size = w;
      Display_types.window_background = c0;
      Display_types.sub_window_sizes = s0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,@[<hv 2>comps_direction =";
    printer_comps_direction Og_precedence.Ck_record_field ppf c;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>scaling_factor =";
    printer_scaling_factor Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>display_image_size =";
    printer_display_image_size Og_precedence.Ck_record_field ppf d;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>window_size =";
    printer_window_size Og_precedence.Ck_record_field ppf w;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>window_background =";
    printer_color Og_precedence.Ck_record_field ppf c0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>sub_window_sizes =";
    printer_sub_window_sizes Og_precedence.Ck_record_field ppf s0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_quantity _ctx ppf = function
  | Display_types.Param ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Param"
  | Display_types.Grad ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Grad"
;;

let printer_pixmap_iter pp_a ctx ppf = function
  | f_f_z_f_a_s_u_f_a_p_u ->
    Og_printer.printer_Fun
      (Og_printer.printer_Fun
         printer_zone_number
         (Og_printer.printer_Fun
            (printer_sub_parameter pp_a)
            Og_printer.printer_Unit))
      (Og_printer.printer_Fun
         (printer_partition pp_a)
         Og_printer.printer_Unit)
      ctx
      ppf
      f_f_z_f_a_s_u_f_a_p_u

and printer_colormap pp_a ctx ppf = function
  | f_a_c -> Og_printer.printer_Fun pp_a printer_color ctx ppf f_a_c
;;

let printer_pixmap ctx ppf = function
  | g_p -> Generic_image_print.printer_pixmap ctx ppf g_p

and printer_lower_left_corner ctx ppf = function
  | p -> printer_point ctx ppf p
;;

let printer_print_pixmap ctx ppf = function
  | f_f_f_u ->
    Og_printer.printer_Fun
      Format_print.printer_formatter
      Og_printer.printer_Unit
      ctx
      ppf
      f_f_f_u

and printer_alter_pixmap pp_a ctx ppf = function
  | f_a_p_f_a_c_f_a_p_u ->
    Og_printer.printer_Fun
      (printer_pixmap_iter pp_a)
      (Og_printer.printer_Fun
         (printer_colormap pp_a)
         (Og_printer.printer_Fun
            (printer_partition pp_a)
            Og_printer.printer_Unit))
      ctx
      ppf
      f_a_p_f_a_c_f_a_p_u

and printer_draw_pixmap ctx ppf = function
  | f_u_u ->
    Og_printer.printer_Fun Og_printer.printer_Unit Og_printer.printer_Unit
      ctx
      ppf
      f_u_u
;;

(* Defining top printers. *)

let print_length ppf = printer_length Og_precedence.Ck_toplevel ppf

and print_width ppf = printer_width Og_precedence.Ck_toplevel ppf

and print_height ppf = printer_height Og_precedence.Ck_toplevel ppf

and print_size ppf = printer_size Og_precedence.Ck_toplevel ppf

and print_tangential ppf = printer_tangential Og_precedence.Ck_toplevel ppf

and print_normal ppf = printer_normal Og_precedence.Ck_toplevel ppf

and print_polar_size ppf = printer_polar_size Og_precedence.Ck_toplevel ppf
;;

let print_color ppf = printer_color Og_precedence.Ck_toplevel ppf
;;

let print_font_name ppf = printer_font_name Og_precedence.Ck_toplevel ppf
;;

let print_aspect ppf = printer_aspect Og_precedence.Ck_toplevel ppf
;;

let print_direction ppf = printer_direction Og_precedence.Ck_toplevel ppf
;;

let print_comps_direction ppf =
  printer_comps_direction Og_precedence.Ck_toplevel ppf
;;

let print_image_size ppf = printer_image_size Og_precedence.Ck_toplevel ppf

and print_font_size ppf = printer_font_size Og_precedence.Ck_toplevel ppf

and print_display_image_size ppf =
  printer_display_image_size Og_precedence.Ck_toplevel ppf

and print_window_size ppf = printer_window_size Og_precedence.Ck_toplevel ppf

and print_sub_window_size ppf =
  printer_sub_window_size Og_precedence.Ck_toplevel ppf

and print_min_window_size ppf =
  printer_min_window_size Og_precedence.Ck_toplevel ppf

and print_max_window_size ppf =
  printer_max_window_size Og_precedence.Ck_toplevel ppf
;;

let print_scaling_factor ppf =
  printer_scaling_factor Og_precedence.Ck_toplevel ppf
;;

let print_scaling ppf = printer_scaling Og_precedence.Ck_toplevel ppf
;;

let print_sub_window_sizes ppf =
  printer_sub_window_sizes Og_precedence.Ck_toplevel ppf
;;

let print_graphics_elements ppf =
  printer_graphics_elements Og_precedence.Ck_toplevel ppf
;;

let print_quantity ppf = printer_quantity Og_precedence.Ck_toplevel ppf
;;

let print_pixmap_iter pp_a ppf =
  printer_pixmap_iter pp_a Og_precedence.Ck_toplevel ppf

and print_colormap pp_a ppf =
  printer_colormap pp_a Og_precedence.Ck_toplevel ppf
;;

let print_pixmap ppf = printer_pixmap Og_precedence.Ck_toplevel ppf

and print_lower_left_corner ppf =
  printer_lower_left_corner Og_precedence.Ck_toplevel ppf
;;

let print_print_pixmap ppf =
  printer_print_pixmap Og_precedence.Ck_toplevel ppf

and print_alter_pixmap pp_a ppf =
  printer_alter_pixmap pp_a Og_precedence.Ck_toplevel ppf

and print_draw_pixmap ppf = printer_draw_pixmap Og_precedence.Ck_toplevel ppf
;;

(* End of OCamlGen, 4.2+dev0 generated code. *)
