(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: parameter.ml,v 1.10 2017/11/07 20:57:41 fclement Exp $ *)

(* Shortcuts for the Mapping library. *)

module Mset = Umatrix_nonempty_set;;
module Mg = Umatrix_graph;;

module Set = Mset;;
module Map = Mg;;

(* Shortcuts for the Refinement library. *)

open Refinement_types;;

(* Parameter operations. *)

let same_geometry = Map.same_domain;;

(* Converters. *)

let sub f p map =
  let dom = map.Map.domain in
  let ff =
    match map.Map.images with
    | Map.Constant y ->
        (fun _idx uidx ->
          f (Map.image p (Set.full_index_of_uindex uidx)) y)
    | Map.Images _ ->
        (fun idx uidx ->
          f
            (Map.image p (Set.full_index_of_uindex uidx))
            (Map.image map idx)) in
  Map.from_fun dom ff
;;

(* Iterators. *)

let map = Map.map_images;;
let fold = Map.fold_images;;

(* Norms. *)

let norm_sup to_float =
  fold (fun accu y -> max accu (abs_float (to_float y))) 0.
;;

(* Domain operations. *)

let bounds_of_domain dom =
  let box { ll = (imin, jmin); ur = (imax, jmax); } idx _ =
    let (i, j) = Set.element dom idx in
    { ll = (min i imin, min j jmin); ur = (max i imax, max j jmax); } in
  let (idx, _) = Set.choice dom in
  let pt = Set.element dom idx in
  Set.fold_left box { ll = pt; ur = pt; } dom
;;
