(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: state_types_spec.mli,v 1.20 2017/11/07 20:57:41 fclement Exp $ *)

open Refinement_types;;
open Optimization_status_types;;
open Stopping_criterion_types;;

module type MAKER =
  functor (Scal : Scalar_spec.SCALAR) ->
  functor (Comps : Components_spec.COMPONENTS) ->
sig
  type scalar = Scal.t;;
  type 'a components = 'a Comps.t;;

  type parameters = scalar parameter components;;
  type gradients = scalar gradient components;;
  type partitions = scalar partition components;;
  type zonations = scalar zonation components;;

  type 'a poly_state =
    {
     stopping_criterion : 'a;
     zonations : zonations;
     cost : cost;
    }
  ;;

  type state = stopping_criterion poly_state
  (** A [state] is the association of a stopping criterion, an
   homogeneous tuple of scalar zonations with the corresponding cost
   function, or exact indicator. *)
  ;;

  type selected_refinement = optimization_status poly_state
  (** A [selected_refinement] is intended to be a candidate [state] with no
   first order indicator computed (nonempty list in the to_do field of delayed
   graphs; in practice, it will be the list of all components). *)
  ;;
end
;;
