(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2013-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: translate_spec.mli,v 1.13 2017/11/07 20:57:41 fclement Exp $ *)

(** {3 Interface to the external part of State maker functor}

 Translation between values of the specific types used in the specialized
 algorithm and the more general types of values that will travel on wires.

 Injection: from specific types to wire types.
 Projection: from wire types to specific types. *)

open Refinement_types;;

module type MAKER =
  functor (Scal : Scalar_spec.SCALAR) ->
  functor (Comps : Components_spec.COMPONENTS) ->
sig
  module Stt : module type of (State_types.Make (Scal) (Comps));;

  type scalar = Scal.t;;
  type 'a components = 'a Comps.t;;

  type gradients = Stt.gradients;;
  type partitions = Stt.partitions;;
  type state = Stt.state;;
  type selected_refinement = Stt.selected_refinement;;

  (** {6 Init} *)

  val inj_init : unit -> Wrapper_types.init_in;;
  val proj_constants :
    Wire_types.cost -> Wire_types.fine_gradient -> Constants_types.t
  ;;
  val proj_coarse_init : Wrapper_types.init_out -> state;;

  (** {6 Fine} *)

  val make_fine :
    state ->
    (unit -> Wrapper_types.grad_in) *
    (Wrapper_types.grad_out -> gradients * gradients_norm)
  ;;

  (** {6 Coarse} *)

  val make_coarse :
    state -> partitions ->
    (unit -> Wrapper_types.optim_in) *
    (Wrapper_types.optim_out -> selected_refinement)
  ;;
end
;;
