CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                              Ref-indic                              C
C                                                                     C
C                        Refinement indicators                        C
C                                                                     C
C                       EPI Serena, INRIA Paris                       C
C                                                                     C
C  Copyright (c) 2013-2017 INRIA.                                     C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     $Id: measure.f,v 1.5 2017/05/31 17:27:50 fclement Exp $

      subroutine measure(ncells, ier, emsg_length, error_msg)
      
      implicit none
      
      integer ERROR_UNIT
      parameter(ERROR_UNIT=0)
      integer HYDROGEOLOGY_DATA_UNIT
      parameter(HYDROGEOLOGY_DATA_UNIT=14)
      integer MEASURES_DATA_UNIT
      parameter(MEASURES_DATA_UNIT=15)
      integer OUTPUT_UNIT
      parameter(OUTPUT_UNIT=18)
      
      include 'common.fh'
      
C     Input argument.
      integer ncells
      
C     Output arguments.
      character*80 error_msg
      integer ier, emsg_length
      
C     Local variables.
      integer i, n, n1
      integer ncells1, ndS, ndT
      real*8 S(np_max), T(np_max), junk
      
C     Get the hydrogeology parameters.
      open(HYDROGEOLOGY_DATA_UNIT,
     +     file='hydrogeology_data', status='old',
     +     access='sequential', form='formatted', iostat=ier)
      if (ier .ne. 0) then
         emsg_length = 49
         error_msg =
     +        'Unable to open file "hydrogeology_data" for read.'
         return
      endif
      
      read(HYDROGEOLOGY_DATA_UNIT, *) ncells1
      if (ncells1 .ne. ncells) then
         write(ERROR_UNIT, *) 'wrong size hydrogeology'
         call exit(1)
      endif
      
      ndS = ncells
      write(OUTPUT_UNIT, *) 'ndS = ', ndS
      do i = 1, ncells
         read(HYDROGEOLOGY_DATA_UNIT, *) S(i)
         write(OUTPUT_UNIT, *) 'S(', i, ') = ', S(i)
      enddo
      
      read(HYDROGEOLOGY_DATA_UNIT, *) ncells1
      if (ncells1 .ne. ncells) then
         write(ERROR_UNIT, *) 'wrong size hydrogeology'
         call exit(1)
      endif
      
      ndT = ncells
      write(OUTPUT_UNIT, *) 'ndT = ', ndT
      do i = 1, ncells
         read(HYDROGEOLOGY_DATA_UNIT, *) T(i)
         write(OUTPUT_UNIT, *) 'T(', i, ') = ', T(i)
      enddo
      
      close(HYDROGEOLOGY_DATA_UNIT)
      
C     Simulation.
c     simu ici permet de calculer les mesures, cout n'a pas de sens.
      call simu(ndS, ndT, S, T, junk)
      
C     Set the measures.
      open(MEASURES_DATA_UNIT,
     +     file='synth_measures', status='unknown',
     +     access='sequential', form='formatted', iostat=ier)
      if (ier .ne. 0) then
         emsg_length = 47
         error_msg =
     +        'Unable to open file "synth_measures" for write.'
         return
      endif
      
      write(MEASURES_DATA_UNIT, '(i6)') nt
      write(MEASURES_DATA_UNIT,'(i6)') ncells
      do n = 1,nt
         n1 = 2*n
         do i = 1, ncells
            write(MEASURES_DATA_UNIT, '(e12.5)') u(i, n1)
         enddo
      enddo
      
      close(MEASURES_DATA_UNIT)
      
      return
      end
